% Noise / Denoise
% Loads an image, creates noisy versions of it and finally denoises it
% again by averaging
% input: image-name
%        noiseLevel: 0..1

function denoise(filename, noiseLevel)

if nargin < 2
    help denoise;
    return;
end

%----------------
% read image etc.
im = imread(filename);
im = im2double(im);
im = normalizeImage(im);
subplot(1,2,1);
imshow(im);
maxIndex = 20;

%------------------------------------------------
% create 20 noisy versions by adding random noise
for i= 1: maxIndex
    IM(:,:,i)=im + (rand(size(im))- 0.5)*noiseLevel;    %add noise (pos/neg)
end

%--------------------
% show noisy pictures
i=1;
fprintf('Move mouse over figure to animate.\n');
fprintf('Move mouse far left to end animation\n');
subplot(1,2,2);
while(1)
    [x,y] = ginput(0);          % read mouse without waiting
                                % ginput(0) does not read button.
                                
    imshow(normalizeImage(IM(:,:,i)));
    i = i+1;
    i = mod(i-1,maxIndex)+1;    % modulo in matlab is a bit more complicated
                                % since indices start with 1 instead of 0 !
    
    if x<20
        break;
    end
    pause(0.05);
end

%--------
% denoise
average = sum(IM,3) / maxIndex;

%-----
% show
result = normalizeImage(average);
subplot(1,2,2);
imshow(result);

return; % that's it !



%==============================================================
% normalize image: transform grayvalue range to interval [0..1]
% local function !
function im = normalizeImage(im)
imMin = min (im(:));
imMax = max (im(:));
im = (im - imMin) / (imMax - imMin);
return
